;============================================
; Up-Down Controls
;
; RECIPE FOR TASTY UP-DOWN CONTROLS:
;
; (This is for simple numeric (integer) controls)
;
; Easiest way is to create up-down controls and adjacent edit controls using
; a resource editor. Give the edit control the next lower tab order than the
; up-down control, and the edit control will become the control's "buddy".
; Be sure to set the following two flags for the up-down control:
;
;	o UDS_AUTOBUDDY
;	o UDS_SETBUDDYINT
;
; To set the allowable range for the control, send the UDM_SETRANGE message to
; the up-down control, specifying the maximum in the low word and the minimum
; in the high word of "lParam".
;
; INTERESTING QUIRK: While you can set the range of an up-down control to include
; negative numbers, if you use the ES_NUMBER style for the edit control (to only
; allow numbers to be typed into the control), you cannot type a "-" to make a
; negative number, so using the "spinner" control is the only way to get one.
;
; That's it! You can use GetDlgItemInt() to read the value (of the edit control,
; not the up-down control).
;
;============================================


	include \masm32\include\masm32rt.inc


;============================================
; Defines, macros, prototypes, etc.
;============================================

WinMain			PROTO :DWORD
DialogProc		PROTO hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD

$mainWindowWidth	EQU 500
$mainWindowHeight	EQU 300

;===== Window styles: =====
$mainWinAttrs		EQU WS_OVERLAPPEDWINDOW OR WS_CLIPCHILDREN OR WS_VISIBLE

$TheMenu		EQU 10000
$TheMenuItem		EQU 10002
$TheDialog		EQU 1000
$dlg_Edit		EQU 1001
$dlg_UpDn		EQU 1002
$dlg_OK			EQU 1003

;===== Window background colors: =====
$bkRED			EQU 254
$bkGRN			EQU 243
$bkBLUE			EQU 199

$BkColor		EQU $bkRED OR ($bkGRN SHL 8) OR ($bkBLUE SHL 16)

;============================================
; HERE BE DATA
;============================================
.data

InstanceHandle		HWND ?
MainWinHandle		HWND ?

CCinitStruct		INITCOMMONCONTROLSEX <SIZEOF INITCOMMONCONTROLSEX, ICC_STANDARD_CLASSES>

MainClassName		DB "UpDownTest", 0
MainTitleText		DB "Up-Down Control Test", 0
InfoText		DB "Info", 0
valuemsg		db "Value in edit control is %d", 0


;============================================
; CODE LIVES HERE
;============================================
.code


start:
	INVOKE	GetModuleHandle, NULL
	MOV	InstanceHandle, EAX


	INVOKE	WinMain, EAX
	INVOKE	ExitProcess, EAX

; As per Iczelion's advice, this function doesn't actually have to be called:
; it's just there so the common controls DLL gets loaded:
	INVOKE	InitCommonControlsEx, OFFSET CCinitStruct

;====================================================================
; Mainline proc
;====================================================================

WinMain	PROC hInst:DWORD
	LOCAL	wc:WNDCLASSEX, msg:MSG, brush:HBRUSH, wX:DWORD, wY:DWORD
	LOCAL	hMenu:HMENU


; Create  brush to set background color:
	INVOKE	CreateSolidBrush, $BkColor
	MOV	brush, EAX

; Register class for parent window:
	MOV	wc.cbSize, SIZEOF WNDCLASSEX
	MOV	wc.style,  CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW
	MOV	wc.lpfnWndProc, OFFSET MainWindowProc
	MOV	wc.cbClsExtra, NULL
	MOV	wc.cbWndExtra, NULL
	MOV	EAX, hInst
	MOV	wc.hInstance, EAX
	MOV	EAX, brush
	MOV	wc.hbrBackground, EAX
	MOV	wc.lpszMenuName,  NULL
	MOV	wc.lpszClassName, OFFSET MainClassName
	MOV	wc.hIcon, NULL
	INVOKE	LoadCursor, NULL, IDC_ARROW
	MOV	wc.hCursor, EAX
	MOV	wc.hIconSm, 0
	INVOKE	RegisterClassEx, ADDR wc

	INVOKE	GetSystemMetrics, SM_CXSCREEN
	MOV	EDX, $mainWindowWidth
	CALL	CenterDim
	MOV	wX, EAX
	INVOKE	GetSystemMetrics, SM_CYSCREEN
	MOV	EDX, $mainWindowHeight
	CALL	CenterDim
	MOV	wY, EAX

	INVOKE	LoadMenu, hInst, $TheMenu
	MOV	hMenu, eax

; Create our main window:
	INVOKE	CreateWindowEx, WS_EX_OVERLAPPEDWINDOW, ADDR MainClassName, ADDR MainTitleText,
		$mainWinAttrs, wX, wY, $mainWindowWidth, $mainWindowHeight,
		NULL, hMenu, hInst, NULL
	MOV	MainWinHandle, EAX

;============= Message loop ===================
msgloop:
	INVOKE	GetMessage, ADDR msg, NULL, 0, 0
	OR	EAX, EAX				;EAX = 0 = exit
	JZ	exit99
	INVOKE	TranslateMessage, ADDR msg
	INVOKE	DispatchMessage, ADDR msg
	JMP	msgloop

exit99:	MOV	EAX, msg.wParam
	RET

WinMain		ENDP


;====================================================================
; Main Window Proc
;====================================================================

MainWindowProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD

	MOV	EAX, uMsg
	CMP	EAX, WM_COMMAND
	JE	do_command
	CMP	EAX, WM_CLOSE
	JE	doclose

dodefault:
; use DefWindowProc for all other messages:
	INVOKE	DefWindowProc, hWin, uMsg, wParam, lParam
	RET

do_command:
	CMP	WORD PTR wParam, $TheMenuItem
	JNE	dodefault

do_dialog:
	INVOKE	DialogBoxParam, InstanceHandle, $TheDialog, MainWinHandle,
		ADDR DialogProc, 0
	XOR	EAX, EAX
	RET

doclose:
	INVOKE	PostQuitMessage, NULL
	MOV	EAX, TRUE
	RET


MainWindowProc	ENDP


DialogProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
		local buffer[100]:byte

	MOV	EAX, uMsg
	CMP	EAX, WM_CLOSE
	JE	do_close
	CMP	EAX, WM_INITDIALOG
	JE	do_init
	CMP	EAX, WM_COMMAND
	JE	do_command
dodefault:
	XOR	EAX, EAX
	RET

do_init:
	INVOKE	GetDlgItem, hWin, $dlg_UpDn

; Set up-down range from -15 to 30:
	INVOKE	SendMessage, EAX, UDM_SETRANGE, 0, -15 SHL 16 OR 30
	MOV	EAX, TRUE
	RET

do_command:
	CMP	WORD PTR wParam, IDOK
	JNE	dodefault
do_close:
	INVOKE	GetDlgItemInt, hWin, $dlg_Edit, NULL, TRUE
	INVOKE	wsprintf, ADDR buffer,ADDR valuemsg, EAX
	INVOKE	MessageBox, 0, ADDR buffer, ADDR InfoText, MB_OK

	INVOKE	EndDialog, hWin, 0
	MOV	EAX, TRUE
	RET

DialogProc	ENDP



;====================================================================
; CenterDim
;
; Returns half screen dimension (X or Y) minus half window dimension
;
; On entry,
;	EAX = screen dimension
;	EDX = window dimension
;
; Returns:
;	sdim/2 - wdim/2
;====================================================================

CenterDim	PROC

	SHR	EAX, 1				;divide screen dimension by 2
	SHR	EDX, 1				;divide window dimension by 2
	SUB	EAX, EDX
	RET					;Return w/# in EAX

CenterDim	ENDP


	END	start

